import sys
import os
import math
import logging

from google.cloud import language

from .logger import Log

logger = Log.get_logger(__name__)

class LanguageServiceClient:

    entity_salience_threshold = 0.7
    category_confidence_threshold = 0.3

    def __init__(self, crendentials_file=None):
        if crendentials_file:
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = crendentials_file

        # Initialize google cloud language service client
        self.client = language.LanguageServiceClient()

    def get_tags_from_webpage_content(self, webpage_content, thresholds={}):
        if thresholds.get('entity_salience_threshold'):
            self.entity_salience_threshold = kwargs.get('entity_salience_threshold')
        if thresholds.get('category_confidence_threshold'):
            self.category_confidence_threshold = kwargs.get('category_confidence_threshold')

        entities = []
        entities = self.get_entities_from_content(webpage_content)
        categories = self.get_categories_from_content(webpage_content)
        return list(dict.fromkeys(entities + categories))  # Remove Duplicates

    def get_categories_from_content(self, webpage_content):
        categories = []
        doc_content = '. '.join([webpage_content['title'], webpage_content['description'], webpage_content['text']])
        if sys.getsizeof(doc_content) > 128000:
            max_len = len(doc_content)*128000/sys.getsizeof(doc_content)
            doc_content = doc_content[:math.floor(max_len)]

        document = language.types.Document(
            content = doc_content,
            # language='en',
            type=language.enums.Document.Type.PLAIN_TEXT,
            # type=language.enums.Document.Type.HTML,
            )

        response = self.client.classify_text(document)

        response_categories = response.categories
        logger.debug('         Categories: ')
        for category in response_categories:
            addCategory = False
            if category.confidence > self.category_confidence_threshold:
                addCategory = True
                labels = [label for label in category.name.split('/') if label]
                categories = categories + labels
            logger.debug('            {} {}: {}'.format('X' if not addCategory else ' ', category.name, category.confidence))
        return categories

    def get_entities_from_content(self, webpage_content):
        entities = []
        document = language.types.Document(
            content = '. '.join([webpage_content['title'], webpage_content['description']]),
            # language='en',
            type=language.enums.Document.Type.PLAIN_TEXT,
            )
        response = self.client.analyze_entities(
            document=document,
            encoding_type='UTF32',
            )

        logger.debug('         Entities: ')
        for entity in response.entities:
            addEntity = False
            if entity.salience > self.entity_salience_threshold:
                addEntity = True
                entities.append(entity.name.title())
            logger.debug('            {} {}: {}'.format('X' if not addEntity else ' ', entity.name.title(), entity.salience))
        return entities



import requests
import logging

from bs4 import BeautifulSoup

from .logger import Log

logger = Log.get_logger(__name__)

class Scraper:
    def get_webpage_content(self, url):
        title = ''
        description = ''
        text = ''

        # Make the request and check object type
        r = requests.get(url, headers={'User-Agent': 'Mozilla/5.0 (sanghviharshit/Auto Pocket tagger) Chrome/18 Safari/535.19'})
        # Extract HTML from Response object
        html = r.text
        # Create a BeautifulSoup object from the HTML
        soup = BeautifulSoup(html, 'html5lib')

        # Get title and description
        try:
            if soup.title:
                title = soup.title.get_text()
            elif soup.h1:
                title = soup.h1.get_text()
            logger.info('         Title: {}'.format(title))

            meta = soup.find('meta', attrs={'name': 'description'})
            for tag, value in meta.attrs.items():
                if tag == 'content':
                    description = value
                    break
            if not description:
                if soup.h2:
                    description = soup.h2.get_text()

            logger.info('         Description: {}'.format(description))

        except Exception as e:
            logger.warning('         ({}) Could not find title/description. {}'.format(url, e))
            pass

        text = self.get_clean_text(soup)

        webpage_content = {
            'title': title,
            'description': description,
            'text': text
        }
        return webpage_content

    def get_clean_text(self, soup):
        # kill all script and style elements
        for script in soup(['script', 'style']):
            script.decompose()    # rip it out

        # get body text
        text_body = soup.body.get_text()
        # break into lines and remove leading and trailing space on each
        text_lines = (line.strip() for line in text_body.splitlines())
        # break multi-headlines into a line each
        text_chunks = (phrase.strip() for line in text_lines for phrase in line.split('  '))
        # drop blank lines
        clean_text = '\n'.join(chunk for chunk in text_chunks if chunk)
        return clean_text



import json
import time
import random
import logging

from .pocket_api_client import PocketAPIClient
from .scraper import Scraper
from .language_service_client import LanguageServiceClient
from .logger import Log

logger = Log.get_logger(__name__)

class PocketTaggerException(Exception):
    pass

class PocketTagger:
    pocket_client = None

    def __init__(self, consumer_key=None, access_token=None, gcloud_credentials_file=None):
        if consumer_key != None and access_token != None:
            self.pocket_client = self.get_pocket_client(consumer_key, access_token)
        self.scraper = Scraper()
        self.language_service_client = LanguageServiceClient(gcloud_credentials_file)

    def get_pocket_client(self, consumer_key=None, access_token=None):
        if self.pocket_client:
            return self.pocket_client
        elif consumer_key is None or access_token is None:
            raise PocketTaggerException
        else:
            return PocketAPIClient(consumer_key, access_token)

    def get_articles_from_api(self, *args, **kwargs):
        return self.get_pocket_client().get_articles_data(*args, **kwargs)

    def add_tags_to_articles(self, articles_with_tags):
        self.get_pocket_client().add_tags_to_articles(articles_with_tags)

    def get_articles_from_file(self, fileName):
        try:
            with open(fileName, 'r') as infile:
                articles = json.load(infile)
                return articles
        except Exception as e:
            logger.error('({}) {}'.format(fileName, e))
            return {}

    def save_articles_to_file(self, file_name, articles):
      with open(file_name, 'w') as file_name:
          json.dump(articles, file_name)

    def get_tags_for_articles(self, articles, *args, **kwargs):
        urls = []
        index = 1

        total_articles = len(articles.items())
        if total_articles == 0:
            logger.warning('No articles fetched from Pocket')

        for id, data in articles.items():
            url = data['given_url']
            tags = []

            try:
                logger.info('({}/{}) {}'.format(index, total_articles, url))
                webpage_content = self.scraper.get_webpage_content(url)
                if webpage_content:
                    tags = self.language_service_client.get_tags_from_webpage_content(webpage_content, *args, **kwargs)
            except Exception as e:
                logger.error('         ({}) {}'.format(url, e))

            if tags:
                logger.info('         Tags: {}'.format(', '.join(tags)))
            else:
                logger.warning('         ({}) No Tags found'.format(url))
            data['tags'] = tags
            index += 1
            # time.sleep(5)

        return articles



import subprocess
import sys
import os

from os import path
from io import open
from setuptools import setup

# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='pocket-tagger',
    version='0.1.0',
    description='Tag your pocket articles from getpocket.com automatically using NLP',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='http://github.com/sanghviharshit/pocket_tagger',
    author='Harshit Sanghvi',
    author_email='hello@sanghviharshit.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    license='MIT',
    keywords='getpocket, pocket, api, articles, automatic, suggested, tag, natural language processing, nlp',
    packages=['pocket_tagger'],
    install_requires=['google.cloud', 'pocket', 'requests', 'bs4'],
    project_urls={
        'Bug Reports': 'https://github.com/sanghviharshit/pocket-tagger/issues',
        'Say Thanks!': 'https://saythanks.io/to/sanghviharshit',
        'Source': 'https://github.com/sanghviharshit/pocket-tagger',
    },
)



import json
import logging

from pocket import Pocket, PocketException

from .logger import Log

logger = Log.get_logger(__name__)

class PocketAPIClient:
    pocket_client = None
    def __init__(self, consumer_key, access_token):
        self.pocket_client = Pocket(consumer_key, access_token)

    def get_articles_data(self, *args, **kwargs):
        # Fetch the articles
        try:
            # For list of optional parameters the API supports - https://getpocket.com/developer/docs/v3/retrieve
            response, headers = self.pocket_client.get(*args, **kwargs)
            return response.get('list')
        except PocketException as e:
            print(e)

    def add_tags_to_articles(self, articles_with_tags, replace=False):
        try:
            total_articles = len(articles_with_tags.items())
            if total_articles == 0:
                return
            pocket_instance = self.pocket_client
            # Start a bulk operation
            for id, data in articles_with_tags.items():
                pocket_instance = pocket_instance.tags_add(id, data['tags'])

            # and commit
            response, headers = self.pocket_client.commit()
            logger.info('Added the tags to articles.')
        except PocketException as e:
            logger.error(e)
